/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


void UpdateShoutcastStationsList()
{
	if (STEAMWORKS_AVAILABLE()) {
		UpdateUsingSteamWorks();
	} else {
		RLogMessage("[SM-RADIO] Unable to find SteamWorks for auto-updating playlists and keeping everything in sync.");
		RLogMessage("[SM-RADIO] Please download from: https://forums.alliedmods.net/showthread.php?t=229556");
	}
}

/*
 * Updater using SteamWorks API Functions
 *
 * Thanks to KyleS for example code
 */

void UpdateUsingSteamWorks()
{
	Handle hHTTPRequest = INVALID_HANDLE;
	
	if (STEAMWORKS_AVAILABLE() && SteamWorks_IsLoaded())
	{
		if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, g_szShoutCastVersionURL)) != INVALID_HANDLE)
		{
			if (SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnVersionCheckComplete) && SteamWorks_SendHTTPRequest(hHTTPRequest))
			{
				RLogMessage("[SM-RADIO] Running HTTP request using SteamWorks for URL=\"%s\"", g_szShoutCastVersionURL);
			}
			else
			{
				CloseHandle(hHTTPRequest);
				RLogMessage("[SM-RADIO] Error: UpdateUsingSteamWorks() Problem sending HTTP request.");
			}
		}
		else
		{
			RLogMessage("[SM-RADIO] Error creating HTTP request using SteamWorks for URL=\"%s\"", g_szShoutCastVersionURL);
		}
	}
	else
	{
		RLogMessage("[SM-RADIO] Error querying SteamWorks_IsLoaded(). Please check the plugin is installed correctly and functioning.");
	}
	
	return;
}

public void SteamWorks_OnVersionCheckComplete(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode)
{
	char szResponseData[128] = "";
	int iResponseSize = 0;
	
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK) {
		if (SteamWorks_GetHTTPResponseBodySize(hRequest, iResponseSize) && SteamWorks_GetHTTPResponseBodyData(hRequest, szResponseData, iResponseSize)) {
			// Check current playlist against web playlist version
			if (StringToInt(szResponseData) > g_iShoutCastPlaylistVersion)
				InitPlaylistDownload(0);
			else
				RLogMessage("[SM-RADIO] Online playlist matches local one. No need to update.");
		} else {
			RLogMessage("[SM-RADIO] SteamWorks_OnVersionCheckComplete() Error: Unable to read response data");
		}
	}
	else
	{
		RLogMessage("[SM-RADIO] SteamWorks_OnVersionCheckComplete() Error: Response code %d", eStatusCode);
	}
	
	CloseHandle(hRequest);
}

// Download the online playlist
void InitPlaylistDownload (int iClient)
{
	char szShoutCastDownloadURL[128] = "";
	Handle hHTTPRequest = INVALID_HANDLE;
	
	if (STEAMWORKS_AVAILABLE() && SteamWorks_IsLoaded())
	{
		// Make sure to apply standard with an else clause - if nothing is set
		if (g_cVarShoutCastPlayListType.IntValue == SHOUTCAST_PLAYLIST_TYPE_NO_CHILDREN)
			strcopy(szShoutCastDownloadURL, sizeof(szShoutCastDownloadURL), g_szShoutCastPlaylistNoChildrenURL);
		else if (g_cVarShoutCastPlayListType.IntValue == SHOUTCAST_PLAYLIST_TYPE_TOP_500)
			strcopy(szShoutCastDownloadURL, sizeof(szShoutCastDownloadURL), g_szShoutCastPlaylistTop500URL);
		else
			strcopy(szShoutCastDownloadURL, sizeof(szShoutCastDownloadURL), g_szShoutCastPlaylistStandardURL);
		
		if (iClient > 0)
		    CReplyToCommand(iClient, "%T", "Forcing Playlist Download", LANG_SERVER, szShoutCastDownloadURL);
		
		RLogMessage("%T", "Forcing Playlist Download", LANG_SERVER, szShoutCastDownloadURL);
		
		if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, szShoutCastDownloadURL)) != INVALID_HANDLE) {
			if (!SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnPlaylistTransferComplete) || !SteamWorks_SendHTTPRequest(hHTTPRequest)) {
				CloseHandle(hHTTPRequest);
				RLogMessage("[SM-RADIO] Error creating HTTP callbacks using SteamWorks.");
			}
		}
		else
		{
			RLogMessage("[SM-RADIO] Error creating HTTP request using SteamWorks for URL=\"%s\"", g_szShoutCastVersionURL);
		}
	}
	else
	{
		if (iClient > 0)
		    CReplyToCommand(iClient, "%T", "Missing SteamWorks", LANG_SERVER);
		
		RLogMessage("%T", "Missing SteamWorks", LANG_SERVER);
	}
	
}

public void SteamWorks_OnPlaylistTransferComplete(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode)
{
	char szPlaylistPath[PLATFORM_MAX_PATH + 1] = "";
	
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK)
	{
		BuildPath(Path_SM, szPlaylistPath, sizeof(szPlaylistPath), "configs/%s", g_szRadioStationsShoutCastBaseFile);
		
		if (SteamWorks_WriteHTTPResponseBodyToFile(hRequest, szPlaylistPath))
		{
			CPrintToChatAll("[SM-RADIO] ShoutCast station list updated. Please reload the map to apply changes.");
			RLogMessage("[SM-RADIO] ShoutCast station list updated. Please reload the map to apply changes.");
		}
	}
	else
	{
		RLogMessage("[SM-RADIO] Error: Unable to download the updated ShoutCast station list.");
	}
	
	CloseHandle(hRequest);
}

